
#ifndef _pf_
#define _pf_

typedef struct Pffunct { 
    char *s ; 
    struct Pf *pf ; 
    Hook *hook ; 
} Pffunct ; 

typedef union { 
	char *s ; 
	Arr *arr ; 
	Tbl *tbl ; 
	Pffunct *pffunct ; 
} Pfvalue ;

typedef struct Pf { 
    int type ; 
    Pfvalue value ;
    struct Pf *parent ; 
    char *filename ;
    char *filled_out ; 
} Pf ; 

#define PFFILE 		0 	// Top level of Pf -- otherwise same as PFARR
#define PFSTRING 	1	// leaf of pf: simple string
#define PFTBL		2	// ordered Tbl{ list
#define PFARR		3	// Arr{ array
#define PFINVALID 	5	// invalid
#define PFANY		6	// special value used internally in pfget
#define PFFCT 		7	// &function() : has to be evaluated
#define PFPF		8	// argument to pfput to graft pf tree into another tree, sets PFFILE to PFARR
#define PFEXEC_ERROR	-1

#define PFMAX_RECURSION 5
#define MAX_RECURSION 	10 	/* includes multiple threads, too*/
#define PF_INTERPOLATE_CHAR '&'

#ifdef  __cplusplus
extern "C" {
#endif

extern Pf * getPf ( char *name );
extern void putPf ( char *name, Pf *pf );
extern void putPf_nofree (char *name, Pf *pf ) ;
extern int updatePf ( char *name );
extern int freePf ( char *name );
extern Pf * pfinterpret ( Pf *pf );
extern int pfconfig ( char *name, void *value );
extern int pfeval ( Pf *pf, char *name, char **value );
extern int pfget ( Pf *pf, char *name, void **result );
extern double pfget_double ( Pf *pf, char *name );
extern double pfget_time  (Pf *pf, char *name) ;
extern long pfget_int ( Pf *pf, char *name );
extern int pfget_boolean ( Pf *pf, char *name );
extern char * pfget_string ( Pf *pf, char *name );
extern Tbl * pfget_tbl ( Pf *pf, char *name );
extern Arr * pfget_arr ( Pf *pf, char *name );
extern double pfget_size ( Pf *pf, char *name );

extern long pfmaxtbl ( Pf *pf );
extern Tbl * pfkeys ( Pf *pf );

extern Pf * pfdup ( Pf * old );
extern void pffree ( Pf *pf );
extern int pfload ( char *envname, char *dirname, char *filename, Pf **pf );
extern Tbl *pffiles ( char *name, int all );
extern int pfread ( char *name, Pf **pf );
extern int pfupdate ( char *name, Pf **pf );
extern int pfinfd ( int fd, Pf **pf );
extern int pfin ( FILE *file, Pf **pf );
extern int pfcompile ( char *string, Pf **pf );
extern int pfcompile_counted ( char *string, long count, Pf **pf );

extern int pfout ( FILE *file, Pf *pf );
extern int pfwrite ( char *filename, Pf *pf );
extern char * pf2string ( Pf *pf );

extern Pf *pfnew ( int type );
extern Pf * existsPf ( char *name );
extern void pfput ( Pf *pf, char *name, void *value, int type );
extern void pfput_double ( Pf *pf, char *name, double val );
extern void pfput_int ( Pf *pf, char *name, long val );
extern void pfput_time ( Pf * pf, char *name, double val );
extern void pfput_boolean ( Pf *pf, char *name, int val );
extern void pfput_string ( Pf *pf, char *name, char *value );
extern void pfput_arr ( Pf *pf, char *name, Arr *value );
extern void pfput_tbl ( Pf *pf, char *name, Tbl *value );

extern Pf * pfdel ( Pf *pf, char *name );

extern void pfvalue ( Pf *pf, char *name, Pf **result );
extern int pfresolve ( Pf *pf, char *name, int create, Pf **value_pf );
extern int pfset ( Pf *pf, char *name, char *value );

extern int pfcmp ( Pf *pf1, Pf *pf2 );
extern int pfrequire ( char *pfname, char *atime ) ;
extern char *pfexpand ( Pf *value ) ;
extern double sz2dbl( char *size) ;
extern void pfprint ( char *pfname, char *entry );


#ifdef	__cplusplus
}
#endif

#endif

/* $Id$ */
